import { of } from 'rxjs';
import { catchError, map, switchMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { PiService } from '@ucap/ng-pi';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';

import { logout, logoutSuccess } from './actions';

@Injectable()
export class Effects {
  logout$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(logout),
      switchMap(action => {
        return this.authenticationProtocolService.logout({}).pipe(
          map(res => logoutSuccess({ res })),
          catchError(error => of(error))
        );
      })
    );
  });

  constructor(
    private actions$: Actions,
    private piService: PiService,
    private authenticationProtocolService: AuthenticationProtocolService
  ) {}
}
