import { createAction, props } from '@ngrx/store';

import { LocaleCode } from '@ucap/core';

import {
  RoomInfo,
  UserInfo as RoomUserInfo,
  InfoRequest as RoomRequest,
  InviteNotification,
  OpenRequest as CreateRequest,
  OpenResponse as CreateResponse,
  Open3Request as CreateTimerRequest,
  Open3Response as CreateTimerResponse,
  ExitRequest as DeleteRequest,
  ExitResponse as DeleteResponse,
  UpdateRequest,
  UpdateResponse,
  InviteRequest,
  InviteResponse,
  ExitForcingRequest,
  ExitForcingResponse,
  UpdateTimerSetRequest,
  UpdateTimerSetResponse
} from '@ucap/protocol-room';

/**
 * retrieve list of room information and list of user information per room
 */
export const rooms = createAction(
  '[ucap::chat::room] rooms',
  props<{ localeCode: LocaleCode }>()
);
/**
 * Success of rooms request
 */
export const roomsSuccess = createAction(
  '[ucap::chat::room] rooms Success',
  props<{
    roomList: RoomInfo[];
    roomUserInfoMap: {
      [param: string]: RoomUserInfo[];
    };
    syncDate: string;
  }>()
);
/**
 * Failure of rooms request
 */
export const roomsFailure = createAction(
  '[ucap::chat::room] rooms Failure',
  props<{ error: any }>()
);

/**
 * retrieve room information and list of user information
 */
export const room = createAction(
  '[ucap::chat::room] room',
  props<{ req: RoomRequest }>()
);
/**
 * Success of room request
 */
export const roomSuccess = createAction(
  '[ucap::chat::room] room Success',
  props<{
    roomInfo: RoomInfo;
    userInfoList: RoomUserInfo[];
  }>()
);
/**
 * Failure of room request
 */
export const roomFailure = createAction(
  '[ucap::chat::room] room Failure',
  props<{ error: any }>()
);

/**
 * create room
 */
export const create = createAction(
  '[ucap::chat::room] create',
  props<{ req: CreateRequest }>()
);
/**
 * Success of create request
 */
export const createSuccess = createAction(
  '[ucap::chat::room] create Success',
  props<{ res: CreateResponse }>()
);
/**
 * Failure of create request
 */
export const createFailure = createAction(
  '[ucap::chat::room] create Failure',
  props<{ error: any }>()
);

/**
 * create timer room
 */
export const createTimer = createAction(
  '[ucap::chat::room] createTimer',
  props<{ req: CreateTimerRequest }>()
);
/**
 * Success of openTimer request
 */
export const createTimerSuccess = createAction(
  '[ucap::chat::room] createTimer Success',
  props<{ res: CreateTimerResponse }>()
);
/**
 * Failure of createTimer request
 */
export const createTimerFailure = createAction(
  '[ucap::chat::room] createTimer Failure',
  props<{ error: any }>()
);

/**
 * exit from room
 */
export const del = createAction(
  '[ucap::chat::room] del',
  props<{ req: DeleteRequest }>()
);
/**
 * Success of del request
 */
export const delSuccess = createAction(
  '[ucap::chat::room] del Success',
  props<{ res: DeleteResponse }>()
);
/**
 * Failure of del request
 */
export const delFailure = createAction(
  '[ucap::chat::room] del Failure',
  props<{ error: any }>()
);

/**
 * update room information
 */
export const update = createAction(
  '[ucap::chat::room] update',
  props<{ req: UpdateRequest }>()
);
/**
 * Success of update request
 */
export const updateSuccess = createAction(
  '[ucap::chat::room] update Success',
  props<{
    res: UpdateResponse;
  }>()
);
/**
 * Failure of update request
 */
export const updateFailure = createAction(
  '[ucap::chat::room] update Failure',
  props<{ error: any }>()
);

/**
 * update isJoinRoom of user information from true to false
 */
export const excludeUser = createAction(
  '[ucap::chat::room] excludeUser',
  props<{ roomId: string; userSeqs: string[] }>()
);
/**
 * Success of excludeUser request
 */
export const excludeUserSuccess = createAction(
  '[ucap::chat::room] excludeUser Success',
  props<{ roomId: string; userSeqs: string[] }>()
);
/**
 * Failure of excludeUser request
 */
export const excludeUserFailure = createAction(
  '[ucap::chat::room] excludeUser Failure',
  props<{ error: any }>()
);

/**
 * open room
 */
export const open = createAction(
  '[ucap::chat::room] open',
  props<{ roomIds: string[] }>()
);
/**
 * Success of open request
 */
export const openSuccess = createAction(
  '[ucap::chat::room] open Success',
  props<{ roomIds: string[] }>()
);
/**
 * Failure of open request
 */
export const openFailure = createAction(
  '[ucap::chat::room] open Failure',
  props<{ error: any }>()
);

/**
 * close room
 */
export const close = createAction(
  '[ucap::chat::room] close',
  props<{ roomIds: string[] }>()
);
/**
 * Success of close request
 */
export const closeSuccess = createAction(
  '[ucap::chat::room] close Success',
  props<{ roomIds: string[] }>()
);
/**
 * Failure of close request
 */
export const closeFailure = createAction(
  '[ucap::chat::room] close Failure',
  props<{ error: any }>()
);

/**
 * Invite conversation partner to room or create room
 */
export const inviteOrCreate = createAction(
  '[ucap::chat::room] inviteOrCreate',
  props<{
    roomInfo: RoomInfo;
    localeCode: LocaleCode;
    req: CreateRequest;
  }>()
);

/**
 * Invite conversation partner to room
 */
export const invite = createAction(
  '[ucap::chat::room] invite',
  props<{ req: InviteRequest; localeCode: LocaleCode }>()
);
/**
 * Success of invite request
 */
export const inviteSuccess = createAction(
  '[ucap::chat::room] invite Success',
  props<{ res: InviteResponse }>()
);
/**
 * Failure of invite request
 */
export const inviteFailure = createAction(
  '[ucap::chat::room] invite Failure',
  props<{ error: any }>()
);

/**
 * Request to ban a conversation partner
 */
export const expel = createAction(
  '[ucap::chat::room] expel',
  props<{ req: ExitForcingRequest }>()
);
/**
 * Success of expel request
 */
export const expelSuccess = createAction(
  '[ucap::chat::room] expel Success',
  props<{ res: ExitForcingResponse }>()
);
/**
 * Failure of expel request
 */
export const expelFailure = createAction(
  '[ucap::chat::room] expel Failure',
  props<{ error: any }>()
);

/**
 * update interval of timer room
 */
export const updateTimeRoomInterval = createAction(
  '[ucap::chat::room] updateTimeRoomInterval',
  props<{ req: UpdateTimerSetRequest }>()
);
/**
 * Success of updateTimeRoomInterval request
 */
export const updateTimeRoomIntervalSuccess = createAction(
  '[ucap::chat::room] updateTimeRoomInterval Success',
  props<{ res: UpdateTimerSetResponse }>()
);
/**
 * Failure of updateTimeRoomInterval request
 */
export const updateTimeRoomIntervalFailure = createAction(
  '[ucap::chat::room] updateTimeRoomInterval Failure',
  props<{ error: any }>()
);

/**
 * norification for invited users
 */
export const inviteNotification = createAction(
  '[ucap::chat::room] invite Notification',
  props<{ noti: InviteNotification; localeCode: LocaleCode }>()
);

/**
 * norification for exited user
 */
export const exitNotification = createAction(
  '[ucap::chat::room] Exit Notification',
  props<{ roomId: string; userSeq: string; senderSeq: string }>()
);
