import { createAction, props } from '@ngrx/store';
import { UserInfo } from '@ucap/protocol-sync';

import {
  AddRequest as BuddyAddRequest,
  AddResponse as BuddyAddResponse,
  DelRequest as BuddyDelRequest,
  DelResponse as BuddyDelResponse,
  UpdateRequest as BuddyUpdateRequest,
  UpdateResponse as BuddyUpdateResponse
} from '@ucap/protocol-buddy';

/**
 * retrieve list of buddy
 */
export const buddy2 = createAction('[ucap::group::buddy] buddy2');
/**
 * Success of buddy2 request
 */
export const buddy2Success = createAction(
  '[ucap::group::buddy] buddy2 Success',
  props<{ buddyList: UserInfo[]; syncDate: string }>()
);
/**
 * Failure of buddy2 request
 */
export const buddy2Failure = createAction(
  '[ucap::group::buddy] buddy2 Failure',
  props<{ error: any }>()
);

/**
 * request to add a buddy
 */
export const add = createAction(
  '[ucap::group::buddy] addBuddy',
  props<{ req: BuddyAddRequest }>()
);
/**
 * Success of add request
 */
export const addSuccess = createAction(
  '[ucap::group::buddy] add Success',
  props<{ res: BuddyAddResponse }>()
);
/**
 * Failure of add request
 */
export const addFailure = createAction(
  '[ucap::group::buddy] add Failure',
  props<{ error: any }>()
);

/**
 * request to delete a buddy
 */
export const del = createAction(
  '[ucap::group::buddy] del',
  props<{ req: BuddyDelRequest }>()
);
/**
 * Success of del request
 */
export const delSuccess = createAction(
  '[ucap::group::buddy] del Success',
  props<{ res: BuddyDelResponse }>()
);
/**
 * Failure of del request
 */
export const delFailure = createAction(
  '[ucap::group::buddy] del Failure',
  props<{ error: any }>()
);

/**
 * 동료 삭제 및 그룹 클리어.(in profile)
 */
export const delAndClear = createAction(
  '[ucap::group::buddy] delAndClear',
  props<{ seq: number }>()
);
/**
 * Success of delAndClear request
 */
export const delAndClearSuccess = createAction(
  '[ucap::group::buddy] delAndClear Success',
  props<{ res: BuddyDelResponse }>()
);
/**
 * Failure of delAndClear request
 */
export const delAndClearFailure = createAction(
  '[ucap::group::buddy] delAndClear Failure',
  props<{ error: any }>()
);

/**
 * 동료 변경(즐겨찾기)
 */
export const update = createAction(
  '[ucap::group::buddy] update',
  props<{ req: BuddyUpdateRequest }>()
);
/**
 * Success of update request
 */
export const updateSuccess = createAction(
  '[ucap::group::buddy] update Success',
  props<{ res: BuddyUpdateResponse }>()
);
/**
 * Failure of update request
 */
export const updateFailure = createAction(
  '[ucap::group::buddy] update Failure',
  props<{ error: any }>()
);
